# combo - content management system
# Copyright (C) 2014-2015  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django import template
from django.conf import settings
from django.db import models
from django.forms import models as model_forms
from django.forms import Select
from django.utils.translation import ugettext_lazy as _

import cmsplugin_blurp.utils

from combo.data.models import CellBase
from combo.data.library import register_cell_class

from .utils import get_wcs_json


def is_wcs_enabled(cls):
    return not hasattr(settings, 'WCS_CELLS_ENABLE') or settings.WCS_CELLS_ENABLE

@register_cell_class
class WcsFormCell(CellBase):
    formdef_reference = models.CharField(_('Form'), max_length=100)

    cached_title = models.CharField(_('Title'), max_length=50)
    cached_url = models.URLField(_('URL'))

    is_enabled = classmethod(is_wcs_enabled)

    class Meta:
        verbose_name = _('Form Link')

    def get_default_form_class(self):
        from .forms import WcsFormCellForm
        return WcsFormCellForm

    def save(self, *args, **kwargs):
        if self.formdef_reference:
            wcs_key, form_slug = self.formdef_reference.split(':')
            wcs_site = settings.COMBO_WCS_SITES.get(wcs_key)
            forms_response_json = get_wcs_json(wcs_site.get('url') + 'json')
            for form in forms_response_json:
                slug = form.get('slug')
                if slug == form_slug:
                    self.cached_title = form.get('title')
                    self.cached_url = form.get('url')
        return super(WcsFormCell, self).save(*args, **kwargs)

    def render(self, context):
        formdef_template = template.loader.get_template('combo/wcs/form.html')
        context['slug'] = self.formdef_reference.split(':')[-1]
        context['title'] = self.cached_title
        context['url'] = self.cached_url
        return formdef_template.render(context)

    def get_additional_label(self):
        if not self.cached_title:
            return
        return self.cached_title


class WcsCommonCategoryCell(CellBase):
    is_enabled = classmethod(is_wcs_enabled)
    category_reference = models.CharField(_('Category'), max_length=100)

    cached_title = models.CharField(_('Title'), max_length=50)
    cached_description = models.TextField(_('Description'), blank=True)
    cached_url = models.URLField(_('Cached URL'))

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        if self.category_reference:
            wcs_key, category_slug = self.category_reference.split(':')
            wcs_site = settings.COMBO_WCS_SITES.get(wcs_key)
            categories_response_json = get_wcs_json(wcs_site.get('url') + 'categories')
            for category in categories_response_json.get('data'):
                slug = category.get('slug')
                if slug == category_slug:
                    self.cached_title = category.get('title')
                    self.cached_description = category.get('description') or ''
                    self.cached_url = category.get('url')
        return super(WcsCommonCategoryCell, self).save(*args, **kwargs)

    def get_additional_label(self):
        if not self.cached_title:
            return
        return self.cached_title


@register_cell_class
class WcsCategoryCell(WcsCommonCategoryCell):
    link_page = models.ForeignKey('data.Page', related_name='link', null=True)

    class Meta:
        verbose_name = _('Category Link')

    def get_default_form_class(self):
        from .forms import WcsCategoryCellForm
        return WcsCategoryCellForm

    def render(self, context):
        category_template = template.loader.get_template('combo/wcs/category.html')
        context['slug'] = self.category_reference.split(':')[-1]
        context['title'] = self.cached_title
        context['description'] = self.cached_description
        if self.link_page:
            context['url'] = self.link_page.get_online_url()
        else:
            context['url'] = self.cached_url
        return category_template.render(context)


class WcsBlurpMixin(object):
    is_enabled = classmethod(is_wcs_enabled)

    def get_blurp_renderer(self, context):
        if self.wcs_site:
            wcs_site = settings.COMBO_WCS_SITES.get(self.wcs_site)
        else:
            wcs_site = settings.COMBO_WCS_SITES.values()[0]

        source = {
            'slug': self.variable_name,
            'parser_type': 'json',
            'verify_certificate': False,
            'allow_redirects': False,
        }

        url = wcs_site.get('url')
        url += self.api_url + '?format=json'
        if context.get('user') and hasattr(context['user'], 'email') and context['user'].email:
            url += '&orig=%s' % wcs_site.get('orig')
            url += '&email=' + context['user'].email
            auth_mech = 'hmac-sha1'
            source['auth_mech'] = auth_mech
            source['signature_key'] = wcs_site.get('secret')

        source['url'] = url

        renderer = cmsplugin_blurp.utils.create_renderer(self.variable_name, {
            'name': self._meta.verbose_name,
            'class': 'cmsplugin_blurp.renderers.data_source.Renderer',
            'sources': [source],
            'template_name': self.template_name,
            'refresh': 0,
            'ajax': False,
        })

        return renderer


class WcsDataBaseCell(CellBase, WcsBlurpMixin):
    is_enabled = classmethod(is_wcs_enabled)
    wcs_site = models.CharField(_('Site'), max_length=50)

    class Meta:
        abstract = True

    def get_default_form_class(self):
        if len(settings.COMBO_WCS_SITES) == 1:
            return None
        combo_wcs_sites = [(x, y.get('title')) for x, y in settings.COMBO_WCS_SITES.items()]
        return model_forms.modelform_factory(self.__class__,
                fields=['wcs_site'],
                widgets={'wcs_site': Select(choices=combo_wcs_sites)})

    def render(self, context):
        renderer = self.get_blurp_renderer(context)
        template = renderer.render_template()
        context = renderer.render(context)
        return template.render(context)


@register_cell_class
class WcsCurrentFormsCell(WcsDataBaseCell):
    api_url = 'myspace/forms'
    variable_name = 'current_forms'
    template_name = 'combo/wcs/current_forms.html'

    class Meta:
        verbose_name = _('Current Forms')


@register_cell_class
class WcsCurrentDraftsCell(WcsDataBaseCell):
    api_url = 'myspace/drafts'
    variable_name = 'current_drafts'
    template_name = 'combo/wcs/current_drafts.html'

    class Meta:
        verbose_name = _('Current Drafts')


@register_cell_class
class WcsFormsOfCategoryCell(WcsCommonCategoryCell, WcsBlurpMixin):
    ordering = models.CharField(_('Order'), max_length=20,
            default='', blank=True,
            choices=[('', _('Default')),
                     ('alpha', _('Alphabetical')),
                     ('popularity', _('Popularity'))])
    limit = models.PositiveSmallIntegerField(_('Limit'),
            null=True, blank=True)

    class Meta:
        verbose_name = _('Forms of Category')

    variable_name = 'forms'
    template_name = 'combo/wcs/forms_of_category.html'

    def get_default_form_class(self):
        from .forms import WcsFormsOfCategoryCellForm
        return WcsFormsOfCategoryCellForm

    @property
    def wcs_site(self):
        return self.category_reference.split(':')[0]

    @property
    def api_url(self):
        return self.category_reference.split(':')[1] + '/json'

    def render(self, context):
        renderer = self.get_blurp_renderer(context)
        template = renderer.render_template()
        context = renderer.render(context)
        context['slug'] = self.category_reference.split(':')[-1]
        context['title'] = self.cached_title
        context['description'] = self.cached_description
        if self.ordering == 'alpha':
            context['forms'] = sorted(context['forms'], lambda x, y: cmp(x.get('title'), y.get('title')))
        elif self.ordering == 'popularity':
            context['forms'] = sorted(context['forms'], lambda x, y: -cmp(x.get('count'), y.get('count')))

        if self.limit:
            if len(context['forms']) > self.limit:
                context['more_forms'] = True
            context['forms'] = context['forms'][:self.limit]

        return template.render(context)
